﻿namespace EditorPartDemo.EditorParts
{
    using EditorPartDemo.Models;
    using Microsoft.SharePoint;
    using System.Collections.Generic;
    using System.Linq;
    using System.Web.UI.WebControls;
    using System.Web.UI.WebControls.WebParts;

    public class MyDocumentsSettings : EditorPart
    {
        #region Fields

        private Panel settingsPanel;

        private CheckBoxList sitesCBL;

        #endregion

        #region Public Methods

        public override bool ApplyChanges()
        {
            try
            {
                this.EnsureChildControls();
                this.ApplyOrSyncChanges("Apply");
            }
            catch
            {
                throw;
            }

            return true;
        }

        public override void SyncChanges()
        {
            try
            {
                this.EnsureChildControls();
                this.ApplyOrSyncChanges();
            }
            catch
            {
                throw;
            }
        }

        #endregion

        #region Protected Methods

        protected override void CreateChildControls()
        {
            this.Controls.Clear();
            this.settingsPanel = new Panel();
            this.sitesCBL = new CheckBoxList();
            this.sitesCBL.Items.AddRange(this.GetUrls().ToArray());
            using (MyDocuments.MyDocuments webPart = (MyDocuments.MyDocuments)this.WebPartToEdit)
            {
                if (webPart != null && webPart.SiteUrls.Count == 0)
                {
                    this.SetDefaultSettings();
                }
            }

            this.settingsPanel.Controls.Add(this.sitesCBL);
            this.Controls.Add(this.settingsPanel);
            this.ApplyOrSyncChanges();
            base.CreateChildControls();
        }

        #endregion

        #region Private Methods

        private List<ListItem> GetUrls()
        {
            List<ListItem> listItems = new List<ListItem>();
            try
            {
                using (SPSite site = new SPSite("http://vpntest:22000/"))
                {
                    SPWebCollection webs = site.RootWeb.GetSubwebsForCurrentUser();
                    List<SPWeb> spWebs = webs.ToList();
                    spWebs.Add(site.RootWeb);
                    for (int i = 0; i < spWebs.Count; i++)
                    {
                        using (SPWeb web = spWebs[i])
                        {
                            SPListCollection lists = web.GetListsOfType(SPBaseType.DocumentLibrary);
                            foreach (SPList list in lists)
                            {
                                if (list.BaseTemplate == SPListTemplateType.DocumentLibrary && list.Title != "Style Library")
                                {
                                    List<SPListItem> total =
                                        (from SPListItem item in list.Items
                                         where item["Created By"].ToString() == SPContext.Current.Web.CurrentUser.ID + ";#" + SPContext.Current.Web.CurrentUser.Name
                                         select item).ToList();
                                    if (total.Any())
                                    {
                                        ListItem listItem = new ListItem
                                        {
                                            Text = web.Title,
                                            Value = web.ID.ToString()
                                        };

                                        listItem.Attributes["Title"] = web.Url;
                                        listItems.Add(listItem);

                                        break;
                                    }
                                }
                            }
                        }
                    }
                }
            }
            catch
            {
                throw;
            }

            return listItems;
        }

        private void ApplyOrSyncChanges(string mode = "Sync")
        {
            try
            {
                using (MyDocuments.MyDocuments webPart = (MyDocuments.MyDocuments)this.WebPartToEdit)
                {
                    if (webPart != null)
                    {
                        if (mode == "Sync")
                        {
                            for (int i = 0; i < webPart.SiteUrls.Count; i++)
                            {
                                for (int j = 0; j < this.sitesCBL.Items.Count; j++)
                                {
                                    // SPWeb's ID is stored in Value property
                                    if (webPart.SiteUrls[i].Value == this.sitesCBL.Items[j].Value)
                                    {
                                        this.sitesCBL.Items[j].Selected = webPart.SiteUrls[i].Selected;

                                        break;
                                    }
                                }
                            }
                        }
                        else
                        {
                            for (int j = 0; j < this.sitesCBL.Items.Count; j++)
                            {
                                for (int i = 0; i < webPart.SiteUrls.Count; i++)
                                {
                                    // SPWeb's ID is stored in Value property
                                    if (webPart.SiteUrls[i].Value == this.sitesCBL.Items[j].Value)
                                    {
                                        webPart.SiteUrls[i].Selected = this.sitesCBL.Items[j].Selected;
                                        break;
                                    }
                                }
                            }
                        }
                    }
                }
            }
            catch
            {
                throw;
            }
        }

        private void SetDefaultSettings()
        {
            try
            {
                using (MyDocuments.MyDocuments webPart = (MyDocuments.MyDocuments)this.WebPartToEdit)
                {
                    for (int i = 0; i < this.sitesCBL.Items.Count; i++)
                    {
                        this.sitesCBL.Items[i].Selected = true;
                        webPart.SiteUrls.Add(new CustomListItemModel
                        {
                            Selected = true,
                            Value = this.sitesCBL.Items[i].Value,
                            WebUrl = this.sitesCBL.Items[i].Attributes["Title"]
                        });
                    }
                }
            }
            catch
            {
                throw;
            }
        }

        #endregion
    }
}
