﻿namespace EditorPartDemo.MyDocuments
{
    using EditorPartDemo.EditorParts;
    using EditorPartDemo.Models;
    using Microsoft.SharePoint;
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Linq;
    using System.Web.UI.WebControls.WebParts;

    [ToolboxItemAttribute(false)]
    public partial class MyDocuments : WebPart
    {
        #region Constructors

        // Uncomment the following SecurityPermission attribute only when doing Performance Profiling on a farm solution
        // using the Instrumentation method, and then remove the SecurityPermission attribute when the code is ready
        // for production. Because the SecurityPermission attribute bypasses the security check for callers of
        // your constructor, it's not recommended for production purposes.
        // [System.Security.Permissions.SecurityPermission(System.Security.Permissions.SecurityAction.Assert, UnmanagedCode = true)]
        public MyDocuments()
        {
            this.SiteUrls = new List<CustomListItemModel>();
        }

        #endregion

        #region Properties

        [WebBrowsable(false), Personalizable(PersonalizationScope.Shared)]
        public List<CustomListItemModel> SiteUrls { get; set; }

        public override object WebBrowsableObject
        {
            get
            {
                return this;
            }
        }

        public List<MyDocumentsModel> MyDocumentsData { get; set; }

        #endregion

        #region Public Methods

        public override EditorPartCollection CreateEditorParts()
        {
            EditorPartCollection editorPartCollection = null;
            try
            {
                EditorPart[] editorParts = new EditorPart[] { new MyDocumentsSettings() };
                editorParts[0].ID = this.ID + "SettingsEditorPart";
                editorParts[0].Title = "Custom Settings";
                editorPartCollection = new EditorPartCollection(base.CreateEditorParts(), editorParts);
            }
            catch
            {
                throw;
            }

            return editorPartCollection;
        }

        #endregion

        #region Protected Methods

        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);
            InitializeControl();
        }

        protected void Page_Load(object sender, EventArgs e)
        {
            try
            {
                if (!this.Page.IsPostBack)
                {
                    this.DataBindDocumentsGridView();
                }
            }
            catch
            {
                throw;
            }
        }

        #endregion

        #region Private Methods

        private void DataBindDocumentsGridView()
        {
            try
            {
                this.MyDocumentsData = this.GetDocuments();
                this.DocumentsGridView.DataSource = this.MyDocumentsData;
                this.DocumentsGridView.DataBind();
            }
            catch
            {
                throw;
            }
        }

        private List<MyDocumentsModel> GetDocuments()
        {
            List<MyDocumentsModel> data = null;
            try
            {
                if (this.SiteUrls.All(x => x.Selected == true))
                {
                    data = this.GetAllDocuments();
                }
                else if (this.SiteUrls.All(x => x.Selected == false))
                {
                    return null;
                }
                else
                {
                    data = this.GetDocumentsByFilter();
                }
            }
            catch
            {
                throw;
            }

            return data;
        }

        private List<MyDocumentsModel> GetAllDocuments()
        {
            List<MyDocumentsModel> urls = new List<MyDocumentsModel>();
            try
            {
                using (SPSite site = new SPSite("http://vpntest:22000/"))
                {
                    SPWebCollection webs = site.RootWeb.GetSubwebsForCurrentUser();
                    List<SPWeb> spWebs = webs.ToList();
                    spWebs.Add(site.RootWeb);
                    for (int i = 0; i < spWebs.Count; i++)
                    {
                        using (SPWeb web = spWebs[i])
                        {
                            SPListCollection lists = web.GetListsOfType(SPBaseType.DocumentLibrary);
                            foreach (SPList list in lists)
                            {
                                if (list.BaseTemplate == SPListTemplateType.DocumentLibrary && list.Title != "Style Library")
                                {
                                    List<SPListItem> total =
                                        (from SPListItem item in list.Items
                                         where item["Created By"].ToString() == SPContext.Current.Web.CurrentUser.ID + ";#" + SPContext.Current.Web.CurrentUser.Name
                                         select item).ToList();
                                    foreach (var item in total)
                                    {
                                        urls.Add(new MyDocumentsModel
                                        {
                                            Name = item.Name,
                                            Modified = item["Modified"] != null ? item["Modified"].ToString() : string.Empty
                                        });
                                    }
                                }
                            }
                        }
                    }
                }
            }
            catch
            {
                throw;
            }

            return urls;
        }

        private List<MyDocumentsModel> GetDocumentsByFilter()
        {
            List<MyDocumentsModel> urls = new List<MyDocumentsModel>();
            try
            {
                for (int i = 0; i < this.SiteUrls.Count; i++)
                {
                    if (this.SiteUrls[i].Selected)
                    {
                        using (SPSite site = new SPSite(this.SiteUrls[i].WebUrl))
                        {
                            using (SPWeb web = site.OpenWeb())
                            {
                                SPListCollection lists = web.GetListsOfType(SPBaseType.DocumentLibrary);
                                foreach (SPList list in lists)
                                {
                                    if (list.BaseTemplate == SPListTemplateType.DocumentLibrary && list.Title != "Style Library")
                                    {
                                        List<SPListItem> total =
                                            (from SPListItem item in list.Items
                                             where item["Created By"].ToString() == SPContext.Current.Web.CurrentUser.ID + ";#" + SPContext.Current.Web.CurrentUser.Name
                                             select item).ToList();
                                        foreach (var item in total)
                                        {
                                            urls.Add(new MyDocumentsModel
                                            {
                                                Name = item.Name,
                                                Modified = item["Modified"] != null ? item["Modified"].ToString() : string.Empty
                                            });
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
            catch
            {
                throw;
            }

            return urls;
        }

        #endregion
    }
}
